"use strict";

exports.__esModule = true;
exports.forRequest = forRequest;
exports.forResponse = forResponse;
exports.transformHeadersCaseToRaw = transformHeadersCaseToRaw;

var _transforms = require("./transforms");

// Transformation routine
function transformHeaders(srcHeaders, ctx, transformList, forcedTransforms) {
  const destHeaders = {};

  const applyTransform = function applyTransform(headerName, headers, transforms) {
    const src = headers[headerName];
    const transform = transforms[headerName];
    const dest = transform ? transform(src, ctx) : src;
    if (dest !== void 0) destHeaders[headerName] = dest;
  };

  Object.keys(srcHeaders).forEach(headerName => applyTransform(headerName, srcHeaders, transformList));
  if (forcedTransforms) Object.keys(forcedTransforms).forEach(headerName => applyTransform(headerName, destHeaders, forcedTransforms));
  return destHeaders;
} // API


function forRequest(ctx) {
  return transformHeaders(ctx.req.headers, ctx, _transforms.requestTransforms, _transforms.forcedRequestTransforms);
}

function forResponse(ctx) {
  return transformHeaders(ctx.destRes.headers, ctx, _transforms.responseTransforms, _transforms.forcedResponseTransforms);
}

function transformHeadersCaseToRaw(headers, rawHeaders) {
  const processedHeaders = {};
  const headersNames = Object.keys(headers);

  for (let i = 0; i < rawHeaders.length; i += 2) {
    const rawHeaderName = rawHeaders[i];
    const headerName = rawHeaderName.toLowerCase();
    const headerIndex = headersNames.indexOf(headerName);

    if (headerIndex > -1) {
      processedHeaders[rawHeaderName] = headers[headerName];
      headersNames[headerIndex] = void 0;
    }
  }

  for (var _i = 0, _headersNames = headersNames; _i < _headersNames.length; _i++) {
    const headerName = _headersNames[_i];
    if (headerName !== void 0) processedHeaders[headerName] = headers[headerName];
  }

  return processedHeaders;
}