"use strict";

exports.__esModule = true;
exports.forcedResponseTransforms = exports.responseTransforms = exports.forcedRequestTransforms = exports.requestTransforms = void 0;

var _headers = _interopRequireDefault(require("../xhr/headers"));

var _authorization = _interopRequireDefault(require("../xhr/authorization"));

var urlUtils = _interopRequireWildcard(require("../../utils/url"));

var _url2 = require("url");

var _cookie = require("../../utils/cookie");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*eslint-disable no-unused-vars*/

/*eslint-enable no-unused-vars*/
function skip() {
  return void 0;
}

function skipIfStateSnapshotIsApplied(src, ctx) {
  return ctx.restoringStorages ? void 0 : src;
}

function isCrossDomainXhrWithoutCredentials(ctx) {
  return ctx.isXhr && !ctx.req.headers[_headers.default.withCredentials] && ctx.dest.reqOrigin !== ctx.dest.domain;
}

function transformAuthorizationHeader(src, ctx) {
  if (src.includes(_authorization.default.valuePrefix)) return src.replace(_authorization.default.valuePrefix, '');
  return isCrossDomainXhrWithoutCredentials(ctx) ? void 0 : src;
}

function transformCookieForFetch(src, ctx) {
  const requestCredentials = ctx.req.headers[_headers.default.fetchRequestCredentials];

  switch (requestCredentials) {
    case 'omit':
      return void 0;

    case 'same-origin':
      return ctx.dest.reqOrigin === ctx.dest.domain ? src : void 0;

    case 'include':
      return src;

    default:
      return void 0;
  }
}

function transformCookie(src, ctx) {
  if (ctx.isXhr) return isCrossDomainXhrWithoutCredentials(ctx) ? void 0 : src;else if (ctx.isFetch) return transformCookieForFetch(src, ctx);
  return src;
}

function generateSyncCookie(ctx, parsedServerCookies) {
  parsedServerCookies = parsedServerCookies.filter(cookie => !cookie.httpOnly);
  let syncWithClientCookies = parsedServerCookies.map(cookie => {
    cookie.isServerSync = true;
    cookie.sid = ctx.session.id;
    return (0, _cookie.formatSyncCookie)(cookie);
  });

  if (ctx.parsedClientSyncCookie) {
    const outdatedSyncCookies = ctx.parsedClientSyncCookie.actual.filter(clientCookie => {
      if (clientCookie.isClientSync && !clientCookie.isWindowSync) return true;

      for (var _iterator = parsedServerCookies, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
        var _ref;

        if (_isArray) {
          if (_i >= _iterator.length) break;
          _ref = _iterator[_i++];
        } else {
          _i = _iterator.next();
          if (_i.done) break;
          _ref = _i.value;
        }

        const serverCookie = _ref;
        if ((0, _cookie.isOutdatedSyncCookie)(clientCookie, serverCookie)) return true;
      }

      return false;
    });
    syncWithClientCookies = ctx.parsedClientSyncCookie.outdated.concat(outdatedSyncCookies).map(_cookie.generateDeleteSyncCookieStr).concat(syncWithClientCookies);
  }

  return syncWithClientCookies;
}

function resolveAndGetProxyUrl(url, ctx) {
  url = urlUtils.prepareUrl(url);

  const _parseUrl = (0, _url2.parse)(url),
        host = _parseUrl.host;

  let isCrossDomain = false;
  if (!host) url = (0, _url2.resolve)(ctx.dest.url, url);

  if (ctx.isIframe && ctx.dest.referer) {
    const isCrossDomainLocationBeforeRedirect = !urlUtils.sameOriginCheck(ctx.dest.referer, ctx.dest.url);
    const isCrossDomainLocationAfterRedirect = !urlUtils.sameOriginCheck(ctx.dest.referer, url);
    isCrossDomain = isCrossDomainLocationBeforeRedirect !== isCrossDomainLocationAfterRedirect;
  }

  return ctx.toProxyUrl(url, isCrossDomain, ctx.contentInfo.contentTypeUrlToken);
}

function transformRefreshHeader(src, ctx) {
  return src.replace(/(url=)(.*)$/i, (_match, prefix, url) => prefix + resolveAndGetProxyUrl(url, ctx));
} // Request headers


const requestTransforms = Object.assign({
  'host': (_src, ctx) => ctx.dest.host,
  'referer': (_src, ctx) => ctx.dest.referer || void 0,
  'origin': (src, ctx) => ctx.dest.reqOrigin || src,
  'content-length': (_src, ctx) => ctx.reqBody.length,
  'cookie': skip,
  'if-modified-since': skipIfStateSnapshotIsApplied,
  'if-none-match': skipIfStateSnapshotIsApplied,
  [_headers.default.requestMarker]: skip,
  [_headers.default.withCredentials]: skip,
  [_headers.default.origin]: skip,
  [_headers.default.fetchRequestCredentials]: skip
}, _authorization.default.headers.reduce((obj, header) => {
  obj[header] = transformAuthorizationHeader;
  return obj;
}, {}));
exports.requestTransforms = requestTransforms;
const forcedRequestTransforms = {
  'cookie': (_src, ctx) => transformCookie(ctx.session.cookies.getHeader(ctx.dest.url) || void 0, ctx),
  // NOTE: All browsers except Chrome don't send the 'Origin' header in case of the same domain XHR requests.
  // So, if the request is actually cross-domain, we need to force the 'Origin' header to support CORS. (B234325)
  'origin': (src, ctx) => {
    const force = (ctx.isXhr || ctx.isFetch) && !src && ctx.dest.domain !== ctx.dest.reqOrigin;
    return force ? ctx.dest.reqOrigin : src;
  }
}; // Response headers

exports.forcedRequestTransforms = forcedRequestTransforms;
const responseTransforms = {
  // NOTE: Disable Content Security Policy (see http://en.wikipedia.org/wiki/Content_Security_Policy).
  'content-security-policy': skip,
  'content-security-policy-report-only': skip,
  'x-content-security-policy': skip,
  'x-content-security-policy-report-only': skip,
  'x-webkit-csp': skip,
  // NOTE: Even if we are not able to be authorized, we should prevent showing the native credentials window.
  'www-authenticate': skip,
  // NOTE: We perform CORS checks on our side, so we skip the related headers.
  'access-control-allow-origin': skip,
  // NOTE: Change the transform type if we have an iframe with an image as src,
  // because it was transformed to HTML with the image tag.
  'content-type': (src, ctx) => ctx.contentInfo.isIframeWithImageSrc ? 'text/html' : src,
  'content-length': (src, ctx) => ctx.contentInfo.requireProcessing ? ctx.destResBody.length : src,
  'location': (src, ctx) => {
    // NOTE: The RFC 1945 standard requires location URLs to be absolute. However, most popular browsers
    // accept relative URLs. We transform relative URLs to absolute to correctly handle this situation.
    if (ctx.contentInfo.isRedirect) return resolveAndGetProxyUrl(src, ctx);
    return src;
  },
  'x-frame-options': (src, ctx) => {
    const cspHeader = ctx.destRes.headers['content-security-policy'];
    if (cspHeader && cspHeader.includes('frame-ancestors ')) return void 0;
    if (!src.includes('ALLOW-FROM')) return src;
    src = src.replace('ALLOW-FROM', '').trim();
    const isCrossDomain = ctx.isIframe && !urlUtils.sameOriginCheck(ctx.dest.url, src);
    const proxiedUrl = ctx.toProxyUrl(src, isCrossDomain, ctx.contentInfo.contentTypeUrlToken);
    return 'ALLOW-FROM ' + proxiedUrl;
  },
  'sourcemap': skip,
  'referrer-policy': () => 'unsafe-url',
  'refresh': (src, ctx) => transformRefreshHeader(src, ctx),
  'link': src => {
    if (/[;\s]rel=\s*prefetch/i.test(src)) return void 0;
    return src;
  }
};
exports.responseTransforms = responseTransforms;
const forcedResponseTransforms = {
  'set-cookie': (src, ctx) => {
    let parsedCookies;
    if (src) parsedCookies = ctx.session.cookies.setByServer(ctx.dest.url, src);
    if (!ctx.isPage || ctx.isIframe) return generateSyncCookie(ctx, parsedCookies || []);
    return [];
  }
};
exports.forcedResponseTransforms = forcedResponseTransforms;